/*
 * Copyright 2014 Pierre-Edouard Portier, Mazen Alsarem
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

#ifndef LDPR_H
#define LDPR_H

#define min(a,b) ((a) < (b) ? (a) : (b))
#define max(a,b) ((a) < (b) ? (b) : (a))

/* indexed double struct */
typedef struct 
{
  size_t index; /* original position in the embedding structure */
  double value;
} IDouble;

double 
eucl_norm(size_t n, double const * const v);

int 
compare_idouble_value(void const * a, void const * b);

void 
idouble_sort(IDouble* values, long size);

/* eucl_norm_cols returns an array of the euclidean norms of the columns of m*/
double* 
eucl_norm_cols (DMat const m);

/* Multiply each row k of m by s[k].
 * The size of s must be the number of rows of m.
 * This operation is applied on the results of a SVD. Thus, the parameter m 
 * is a dense matrix. */
DMat 
scale_rows (DMat const m, double const * const s);

#endif /* LDPR_H */
